//
//  NSMutableOrderedSet+SMBlocks.h
//  SMBlocks
//

#import "SMBlocksTypes.h"

/** Block extensions for NSMutableOrderedSet.

 These utilities expound upon the SMBlocks additions to the immutable
 superclass by allowing certain utilities to work on an instance of the mutable
 class, saving memory by not creating an immutable copy of the results.

 Includes code by the following:

 - [Martin Schürrer](https://github.com/MSch)
 - [Zach Waldowski](https://github.com/zwaldowski)

 @see NSOrderedSet(SMBlocks)
 */
@interface NSMutableOrderedSet (SMBlocks)

/** Filters a mutable ordered set to the objects matching the block.

 @param block A single-argument, BOOL-returning code block.
 @see <NSOrderedSet(SMBlocks)>reject:
 */
- (void)performSelect:(SMValidationBlock)block;

/** Filters a mutable ordered set to all objects but the ones matching the
 block, the logical inverse to select:.

 @param block A single-argument, BOOL-returning code block.
 @see <NSOrderedSet(SMBlocks)>select:
 */
- (void)performReject:(SMValidationBlock)block;

/** Transform the objects in the ordered set to the results of the block.

 This is sometimes referred to as a transform, mutating one of each object:
     [foo performMap:^id(id object) {
         return [dateTransformer dateFromString:object];
     }];

 @param block A single-argument, object-returning code block.
 @see <NSOrderedSet(SMBlocks)>map:
 */
- (void)performMap:(SMTransformBlock)block;

@end
