//
//  NSObject+SMObservations.h
//  SMObservation
//
//  Created by Stephan Michels on 23.11.12.
//  Copyright (c) 2012 Stephan Michels. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SMCancellable.h"
#import "SMNotificationObservation.h"
#import "SMBlockObservation.h"

@interface NSObject (SMObservations)

#pragma mark - Observations for notifications

- (void)addObservationWithCenter:(NSNotificationCenter *)center
						 forName:(NSString *)name
						  object:(id)object
					  usingBlock:(SMNoticationObservationBlock)block
						 context:(void *)context;
- (void)addObservationWithCenter:(NSNotificationCenter *)center
						forNames:(NSArray *)names
						  object:(id)object
					  usingBlock:(SMNoticationObservationBlock)block
						 context:(void *)context;

#pragma mark - Observations with blocks

- (void)addObservationWithObject:(id)object
					  forKeyPath:(NSString *)keyPath
						 options:(NSKeyValueObservingOptions)options
					  usingBlock:(SMObservationBlock)block
						 context:(void *)context;
- (void)addObservationWithObject:(id)object
					 forKeyPaths:(NSArray *)keyPaths
						 options:(NSKeyValueObservingOptions)options
					  usingBlock:(SMObservationBlock)block
						 context:(void *)context;

#pragma mark - Observations with delegate

- (void)addObservationWithObject:(id)object
					  forKeyPath:(NSString *)keyPath
						 options:(NSKeyValueObservingOptions)options
						observer:(id)observer
						selector:(SEL)selector
						 context:(void *)context;
- (void)addObservationWithObject:(id)object
					 forKeyPaths:(NSArray *)keyPaths
						 options:(NSKeyValueObservingOptions)options
						observer:(id)observer
						selector:(SEL)selector
						 context:(void *)context;

#pragma mark - Bindinds

- (void)addBindingWithObject:(id)object
					 forName:(NSString *)bindingName
					toObject:(id)targetObject
				 withKeyPath:(NSString *)keyPath
					 options:(NSDictionary *)options
					 context:(void *)context;
- (void)addBindingWithObject:(id)object
					toObject:(id)targetObject
		 forNamesAndKeyPaths:(NSDictionary *)namesAndKeyPaths
					 options:(NSDictionary *)options
					 context:(void *)context;

#pragma mark - Common

- (NSArray *)observationsForContext:(void *)context;

- (void)addObservation:(id<SMCancellable>)observation
			forContext:(void *)context;
- (void)removeAllObservationsForContext:(void *)context;


@end
